######################################################################################################################
#                                                          							              	                             #
#  This R script TransponeMat.r was developed in 20210708 for transposing a Mat to its original transposed Mat       # 
#  by using transMat <- IndMar.to.MarInd.transpose(Mat) with data,table package installed and library(data.table)    #
#	
#     Yang Mo-Hua  July 8, 2021                                                                                                               #
#######################################################################################################################

#----------------------------------------
# 1. setting the environment:
# If there is no data.table, please installed the data.table package with install.packages() and library(data.table)
#-----------------------------------------

# install.packages("data.table", dep=T)
library(data.table)

rm(list=ls())
#--------------------------------------------------------------------
# 2. read the data file into the memory and check the data structure
#---------------------------------------------------------------------
#  Such as the input file was named with "Inputdatafilename.csv" or "Inputdatafilename.txt", 
#  InputIndMar <- fread(input = "Inputdatafilename.csv", header = FALSE, strip.white = FALSE, na.strings = c("NA"), fill =TRUE)  # input data as a data.table for convenient transformation
# If the input data should have its column names and its row names with sequenced numbered IDs. the header = TRUE, otherwise, header=FALSE in the fread() 
#-------------------------------------------------------------------------------------------------- 

#-----------------------------------------------------------------------------------------------------------
# 3. case study, the file name is 5_diploid_SNP_Amova_test_AA5d_SourceFU.csv with column names, header=TRUE
#------------------------------------------------------------------------------------------------------------
MassonSNP2indmat <- fread(input = "2_haploid_AA2hssr711gener.csv", header = TRUE, strip.white = TRUE, na.strings = c("NA"), fill =TRUE)  # input data as a data.table for convenient transformation
MassonSNP2indmat[1:6, 1:7, with=FALSE]
MassonSNP2indmat[, .N]
head(MassonSNP2indmat)
tail(MassonSNP2indmat)


IndMar.to.MarInd.transpose <- function(Ind.Mark){ 
  
  ReCharOrder.rename <- function(Mnchr, Chr1){   # Mt is a set of characters, FF is a character for new order beginning)
    nns <- length(Mnchr)
    nnc <- nchar(nns)
    chtc <- paste(Chr1, sapply(mapply(rep, nnc-nchar(1:nns), MoreArgs=list(x=0)), paste, collapse=''), 1:nns, sep="")
    return(chtc)
  }
  
  DTM1 <- copy(Ind.Mark)
  headname <- names(DTM1)
  hn <- length(headname) 
  Hedname <- ReCharOrder.rename(1:hn, "H") 
  Hehe <- rbind(Hedname, headname)
  HeheDT <- data.table(Hehe)
  HehedmarL <- list( HeheDT, DTM1)
  HehedmarDT <- rbindlist(HehedmarL, use.names=FALSE, fill=FALSE) # update the rbindlist() in use.names=FALSE to bind by position.
  Hehedn <- HehedmarDT[,.N]
  HehedIDrow <- ReCharOrder.rename(1:Hehedn, "Idr")
  HehedIDrowDT <- data.table(HehedIDrow)
  RowIDHeHeMar <- data.table(HehedIDrowDT, HehedmarDT)
  HHhename <- names(RowIDHeHeMar)
  HHhe <- ReCharOrder.rename(1:length(HHhename), "HH")
  setnames(RowIDHeHeMar, HHhe )
  MarHHhe <- transpose(RowIDHeHeMar, fill=NA, ignore.empty=FALSE)
  setnames(MarHHhe, HehedIDrow)
  MarHHhesub <- MarHHhe[-1]
  MHc <- names(MarHHhesub)
  MHcSaveColums <- MHc[-1]
  MarHHhesubFinalTrans <- MarHHhesub[, ..MHcSaveColums]
  SHname <- unlist(MarHHhesubFinalTrans[1])
  MarHHhesubFinalTrans01 <- MarHHhesubFinalTrans[-1]
  setnames(MarHHhesubFinalTrans01, SHname)
  return(MarHHhesubFinalTrans01)
}


#----------------------------------------------------------------------------------------------
# 3. apply the function for transpose mat by using the function of IndMar.to.MarInd.transpose()
#----------------------------------------------------------------------------------------------

TransPosedMat <- IndMar.to.MarInd.transpose(MassonSNP2indmat)
TransPosedMat[1:6, 1:8, with=FALSE]

#----------------------------------------------------------------------------------------------
# 4. Save the transposed mat with a file name, it could be a .txt or .csv format, both are OK!
# When the job was done, check the transposed file in the current working directory.
# Note" There maybe is another line of column names in the transposed mat obtained.
#----------------------------------------------------------------------------------------------

fwrite(TransPosedMat, file="2_haploid_AA2hssr711generTransposeMat01.csv")

#---------------------
#5 Exit the R scripts
#---------------------

# q()


